@@ -12,7 +12,7 @@
  *
  * @package Sabre
  * @subpackage DAV
- * @copyright Copyright (C) 2007-2012 Rooftop Solutions. All rights reserved.
+ * @copyright Copyright (C) 2007-2013 Rooftop Solutions. All rights reserved.
  * @author Evert Pot (http://www.rooftopsolutions.nl/)
  * @license http://code.google.com/p/sabredav/wiki/License Modified BSD License
  */
@@ -23,14 +23,14 @@ class Sabre_DAV_Locks_Plugin extends Sabre_DAV_ServerPlugin {
      *
      * @var Sabre_DAV_Locks_Backend_Abstract
      */
-    private $locksBackend;
+    protected $locksBackend;
 
     /**
      * server
      *
      * @var Sabre_DAV_Server
      */
-    private $server;
+    protected $server;
 
     /**
      * __construct
@@ -152,6 +152,7 @@ class Sabre_DAV_Locks_Plugin extends Sabre_DAV_ServerPlugin {
             case 'MKCOL' :
             case 'PROPPATCH' :
             case 'PUT' :
+            case 'PATCH' :
                 $lastLock = null;
                 if (!$this->validateLock($uri,$lastLock))
                     throw new Sabre_DAV_Exception_Locked($lastLock);
@@ -292,7 +293,10 @@ class Sabre_DAV_Locks_Plugin extends Sabre_DAV_ServerPlugin {
             $this->server->tree->getNodeForPath($uri);
 
             // We need to call the beforeWriteContent event for RFC3744
-            $this->server->broadcastEvent('beforeWriteContent',array($uri));
+            // Edit: looks like this is not used, and causing problems now.
+            //
+            // See Issue 222
+            // $this->server->broadcastEvent('beforeWriteContent',array($uri));
 
         } catch (Sabre_DAV_Exception_NotFound $e) {
 
@@ -614,12 +618,16 @@ class Sabre_DAV_Locks_Plugin extends Sabre_DAV_ServerPlugin {
      * @return Sabre_DAV_Locks_LockInfo
      */
     protected function parseLockRequest($body) {
-
-        $xml = simplexml_load_string($body,null,LIBXML_NOWARNING);
-        $xml->registerXPathNamespace('d','DAV:');
+		$loadEntities = libxml_disable_entity_loader(true);
+        $xml = simplexml_load_string(
+            Sabre_DAV_XMLUtil::convertDAVNamespace($body),
+            null,
+            LIBXML_NOWARNING);
+		libxml_disable_entity_loader($loadEntities);
+        $xml->registerXPathNamespace('d','urn:DAV');
         $lockInfo = new Sabre_DAV_Locks_LockInfo();
 
-        $children = $xml->children("DAV:");
+        $children = $xml->children("urn:DAV");
         $lockInfo->owner = (string)$children->owner;
 
         $lockInfo->token = Sabre_DAV_UUIDUtil::getUUID();
