@@ -8,6 +8,25 @@
 
 Calendar={
 	Util:{
+		sendmail: function(eventId, location, description, dtstart, dtend){
+			Calendar.UI.loading(true);
+			$.post(
+			OC.filePath('calendar','ajax/event','sendmail.php'),
+			{
+				eventId:eventId,
+				location:location,
+				description:description,
+				dtstart:dtstart,
+				dtend:dtend
+			},
+			function(result){
+				if(result.status !== 'success'){
+					OC.dialogs.alert(result.data.message, 'Error sending mail');
+				}
+				Calendar.UI.loading(false);
+			}
+		);
+		},
 		dateTimeToTimestamp:function(dateString, timeString){
 			dateTuple = dateString.split('-');
 			timeTuple = timeString.split(':');
@@ -64,7 +83,6 @@ Calendar={
 		}
 	},
 	UI:{
-		scrollcount: 0,
 		loading: function(isLoading){
 			if (isLoading){
 				$('#loading').show();
@@ -74,7 +92,6 @@ Calendar={
 		},
 		startEventDialog:function(){
 			Calendar.UI.loading(false);
-			$('.tipsy').remove();
 			$('#fullcalendar').fullCalendar('unselect');
 			Calendar.UI.lockTime();
 			$( "#from" ).datepicker({
@@ -109,11 +126,16 @@ Calendar={
 			$('#event').dialog({
 				width : 500,
 				height: 600,
+				resizable: false,
+				draggable: false,
 				close : function(event, ui) {
 					$(this).dialog('destroy').remove();
 				}
 			});
 			Calendar.UI.Share.init();
+			$('#sendemailbutton').click(function() {
+				Calendar.Util.sendmail($(this).attr('data-eventid'), $(this).attr('data-location'), $(this).attr('data-description'), $(this).attr('data-dtstart'), $(this).attr('data-dtend'));
+			})
 		},
 		newEvent:function(start, end, allday){
 			start = Math.round(start.getTime()/1000);
@@ -198,7 +220,6 @@ Calendar={
 				},"json");
 		},
 		moveEvent:function(event, dayDelta, minuteDelta, allDay, revertFunc){
-			$('.tipsy').remove();
 			Calendar.UI.loading(true);
 			$.post(OC.filePath('calendar', 'ajax/event', 'move.php'), { id: event.id, dayDelta: dayDelta, minuteDelta: minuteDelta, allDay: allDay?1:0, lastmodified: event.lastmodified},
 			function(data) {
@@ -213,7 +234,6 @@ Calendar={
 			});
 		},
 		resizeEvent:function(event, dayDelta, minuteDelta, revertFunc){
-			$('.tipsy').remove();
 			Calendar.UI.loading(true);
 			$.post(OC.filePath('calendar', 'ajax/event', 'resize.php'), { id: event.id, dayDelta: dayDelta, minuteDelta: minuteDelta, lastmodified: event.lastmodified},
 			function(data) {
@@ -267,54 +287,10 @@ Calendar={
 			}
 		},
 		showCalDAVUrl:function(username, calname){
-			$('#caldav_url').val(totalurl + '/' + username + '/' + calname);
+			$('#caldav_url').val(totalurl + '/' + encodeURIComponent(username) + '/' + encodeURIComponent(calname));
 			$('#caldav_url').show();
 			$("#caldav_url_close").show();
 		},
-		initScroll:function(){
-			if(window.addEventListener)
-				document.addEventListener('DOMMouseScroll', Calendar.UI.scrollCalendar, false);
-			//}else{
-				document.onmousewheel = Calendar.UI.scrollCalendar;
-			//}
-		},
-		scrollCalendar:function(event){
-			$('#fullcalendar').fullCalendar('option', 'height', $(window).height() - $('#controls').height() - $('#header').height() - 15);
-			$('.tipsy').remove();
-			var direction;
-			if(event.detail){
-				if(event.detail < 0){
-					direction = 'top';
-				}else{
-					direction = 'down';
-				}
-			}
-			if (event.wheelDelta){
-				if(event.wheelDelta > 0){
-					direction = 'top';
-				}else{
-					direction = 'down';
-				}
-			}
-			Calendar.UI.scrollcount++;
-			if(Calendar.UI.scrollcount < 5){
-				return;
-			}
-
-			var scroll = $(document).scrollTop(),
-				doc_height = $(document).height(),
-				win_height = $(window).height();
-			if(direction == 'down'/* && win_height == (doc_height - scroll)*/){
-				$('#fullcalendar').fullCalendar('next');
-				$(document).scrollTop(0);
-				event.preventDefault();
-			}else/* if (direction == 'top' && scroll == 0) */{
-				$('#fullcalendar').fullCalendar('prev');
-				$(document).scrollTop(win_height);
-				event.preventDefault();
-			}
-			Calendar.UI.scrollcount = 0;
-		},
 		repeat:function(task){
 			if(task=='init'){
 				$('#byweekno').multiselect({
@@ -427,14 +403,14 @@ Calendar={
 				case 'month':
 					id = 'onemonthview_radio';
 					break;
-				case 'list':
-					id = 'listview_radio';
+				case 'agendaDay':
+					id = 'onedayview_radio';
 					break;
 			}
 			$('#'+id).addClass('active');
 		},
 		categoriesChanged:function(newcategories){
-			categories = $.map(newcategories, function(v) {return v;});
+			categories = $.map(newcategories, function(v) {return v.name;});
 			console.log('Calendar categories changed to: ' + categories);
 			$('#category').multiple_autocomplete('option', 'source', categories);
 		},
@@ -506,7 +482,12 @@ Calendar={
 			},
 			submit:function(button, calendarid){
 				var displayname = $.trim($("#displayname_"+calendarid).val());
-				var active = $("#edit_active_"+calendarid+":checked").length;
+				//var active = $("#edit_active_"+calendarid+":checked").length;
+				var active =0;
+				if( $("#edit_active_"+calendarid).is(':checked') ){
+					 active =1;
+				}
+				
 				var description = $("#description_"+calendarid).val();
 				var calendarcolor = $("#calendarcolor_"+calendarid).val();
 				if(displayname == ''){
@@ -529,7 +510,7 @@ Calendar={
 							$('#fullcalendar').fullCalendar('removeEventSource', data.eventSource.url);
 							$('#fullcalendar').fullCalendar('addEventSource', data.eventSource);
 							if (calendarid == 'new'){
-								$('#choosecalendar_dialog > table:first').append('<tr><td colspan="6"><a href="#" id="chooseCalendar"><input type="button" value="' + newcalendar + '"></a></td></tr>');
+								$('#calendar > table:first').append('<tr><td colspan="6"><a href="#" id="chooseCalendar"><input type="button" value="' + newcalendar + '"></a></td></tr>');
 							}
 						}else{
 							$("#displayname_"+calendarid).css('background-color', '#FF2626');
@@ -573,7 +554,7 @@ Calendar={
 			init:function(){
 				if(typeof OC.Share !== typeof undefined){
 					var itemShares = [OC.Share.SHARE_TYPE_USER, OC.Share.SHARE_TYPE_GROUP];
-					$('#sharewith').autocomplete({minLength: 2, source: function(search, response) {
+					$('#sharewith').autocomplete({minLength: 1, source: function(search, response) {
 						$.get(OC.filePath('core', 'ajax', 'share.php'), { fetch: 'getShareWith', search: search.term, itemShares: itemShares }, function(result) {
 							if (result.status == 'success' && result.data.length > 0) {
 								response(result.data);
@@ -595,10 +576,12 @@ Calendar={
 							var newitem = '<li data-item-type="event"'
 								+ 'data-share-with="'+shareWith+'" '
 								+ 'data-permissions="'+permissions+'" '
-								+ 'data-share-type="'+shareType+'">'+shareWith+' ('+(shareType == OC.Share.SHARE_TYPE_USER ? t('core', 'user') : t('core', 'group'))+')'
-								+ '<span class="shareactions"><input class="update" type="checkbox" title="'+t('core', 'Editable')+'">'
-								+ '<input class="share" type="checkbox" title="'+t('core', 'Shareable')+'" checked="checked">'
-								+ '<input class="delete" type="checkbox" title="'+t('core', 'Deletable')+'">'
+								+ 'data-share-type="'+shareType+'">'
+								+ shareWith
+								+ (shareType === OC.Share.SHARE_TYPE_GROUP ? ' ('+t('core', 'group')+')' : '')
+								+ '<span class="shareactions">'
+								+ '<label><input class="update" type="checkbox" checked="checked">'+t('core', 'can edit')+'</label>'
+								+ '<label><input class="share" type="checkbox" checked="checked">'+t('core', 'can share')+'</label>'
 								+ '<img class="svg action delete" title="Unshare"src="'+ OC.imagePath('core', 'actions/delete.svg') +'"></span></li>';
 							$('.sharedby.eventlist').append(newitem);
 							$('#sharedWithNobody').remove();
@@ -618,10 +601,9 @@ Calendar={
 						var permission = null;
 						if($(this).hasClass('update')) {
 							permission = OC.PERMISSION_UPDATE;
+							permission = OC.PERMISSION_DELETE;
 						} else if($(this).hasClass('share')) {
 							permission = OC.PERMISSION_SHARE;
-						} else if($(this).hasClass('delete')) {
-							permission = OC.PERMISSION_DELETE;
 						}
 						// This is probably not the right way, but it works :-P
 						if($(this).is(':checked')) {
@@ -629,6 +611,9 @@ Calendar={
 						} else {
 							permissions -= permission;
 						}
+						
+						container.data('permissions',permissions);
+						
 						OC.Share.setPermissions(itemType, itemSource, shareType, shareWith, permissions);
 					});
 	
@@ -857,7 +842,6 @@ function ListView(element, calendar) {
 	}
 }
 $(document).ready(function(){
-	Calendar.UI.initScroll();
 	$('#fullcalendar').fullCalendar({
 		header: false,
 		firstDay: firstDay,
@@ -875,7 +859,7 @@ $(document).ready(function(){
 		titleFormat: {
 			month: t('calendar', 'MMMM yyyy'),
 					// September 2009
-			week: t('calendar', "MMM d[ yyyy]{ '&#8212;'[ MMM] d yyyy}"),
+			week: t('calendar', "MMM d[ yyyy]{ '–'[ MMM] d yyyy}"),
 					// Sep 7 - 13 2009
 			day: t('calendar', 'dddd, MMM d, yyyy'),
 					// Tuesday, Sep 8, 2009
@@ -892,6 +876,11 @@ $(document).ready(function(){
 				$.post(OC.filePath('calendar', 'ajax', 'changeview.php'), {v:view.name});
 				defaultView = view.name;
 			}
+			if(view.name === 'agendaDay') {
+				$('td.fc-state-highlight').css('background-color', '#ffffff');
+			} else{
+				$('td.fc-state-highlight').css('background-color', '#ffc');
+			}
 			Calendar.UI.setViewActive(view.name);
 			if (view.name == 'agendaWeek') {
 				$('#fullcalendar').fullCalendar('option', 'aspectRatio', 0.1);
@@ -911,17 +900,6 @@ $(document).ready(function(){
 		eventResize: Calendar.UI.resizeEvent,
 		eventRender: function(event, element) {
 			element.find('.fc-event-title').text($("<div/>").html(escapeHTML(event.title)).text())
-			element.tipsy({
-				className: 'tipsy-event',
-				opacity: 0.9,
-				gravity:$.fn.tipsy.autoBounds(150, 's'),
-				fade:true,
-				delayIn: 400,
-				html:true,
-				title:function() {
-					return Calendar.UI.getEventPopupText(event);
-				}
-			});
 		},
 		loading: Calendar.UI.loading,
 		eventSources: eventSources
@@ -943,17 +921,21 @@ $(document).ready(function(){
 		}
 	});
 	fillWindow($('#content'));
-	OCCategories.changed = Calendar.UI.categoriesChanged;
-	OCCategories.app = 'calendar';
-	OCCategories.type = 'event';
+
+	$(OC.Tags).on('change', function(event, data) {
+		if(data.type === 'event') {
+			Calendar.UI.categoriesChanged(data.tags);
+		}
+	});
+
 	$('#oneweekview_radio').click(function(){
 		$('#fullcalendar').fullCalendar('changeView', 'agendaWeek');
 	});
 	$('#onemonthview_radio').click(function(){
 		$('#fullcalendar').fullCalendar('changeView', 'month');
 	});
-	$('#listview_radio').click(function(){
-		$('#fullcalendar').fullCalendar('changeView', 'list');
+	$('#onedayview_radio').click(function(){
+		$('#fullcalendar').fullCalendar('changeView', 'agendaDay');
 	});
 	$('#today_input').click(function(){
 		$('#fullcalendar').fullCalendar('today');
