@@ -16,16 +16,13 @@
 "Deletion failed" => "删除失败",
 "ddd d MMMM[ yyyy]{ - [ddd d] MMMM yyyy}" => "ddd d MMMM[ yyyy]{ - [ddd d] MMMM yyyy}",
 "ddd d MMMM[ yyyy] HH:mm{ - [ ddd d MMMM yyyy] HH:mm}" => "ddd d MMMM[ yyyy] HH:mm{ - [ ddd d MMMM yyyy] HH:mm}",
-"user" => "用户",
 "group" => "组",
-"Editable" => "可编辑",
-"Shareable" => "可共享",
-"Deletable" => "可删除",
+"can edit" => "可以修改",
 "ddd" => "ddd",
 "ddd M/d" => "ddd M/d",
 "dddd M/d" => "dddd M/d",
 "MMMM yyyy" => "MMMM yyyy",
-"MMM d[ yyyy]{ '&#8212;'[ MMM] d yyyy}" => "MMM d[ yyyy]{ '&#8212;'[ MMM] d yyyy}",
+"MMM d[ yyyy]{ '–'[ MMM] d yyyy}" => "MMM d[ yyyy]{ '–'[ MMM] d yyyy}",
 "dddd, MMM d, yyyy" => "dddd, MMM d, yyyy",
 "Sunday" => "星期日",
 "Monday" => "星期一",
@@ -98,9 +95,6 @@
 "You do not have the permissions to add events to this calendar." => "您没有权限增加事项到此日历",
 "You do not have the permissions to delete this event." => "您没有权限删除此事件",
 "Busy" => "忙碌",
-"Public" => "公共",
-"Private" => "私人",
-"Confidential" => "证书",
 "Does not repeat" => "不重复",
 "Daily" => "每天",
 "Weekly" => "每周",
@@ -124,47 +118,37 @@
 "by yearday(s)" => "按每年的某天",
 "by weeknumber(s)" => "按星期数",
 "by day and month" => "按天和月份",
+"Contact birthdays" => "联系人生日",
 "Date" => "日期",
 "Cal." => "日历",
 "Week" => "星期",
 "Month" => "月",
-"List" => "列表",
 "Today" => "今天",
 "Settings" => "设置",
-"Your calendars" => "您的日历",
-"CalDav Link" => "CalDav 链接",
 "Share Calendar" => "共享日历",
+"CalDav Link" => "CalDav 链接",
 "Download" => "下载",
 "Edit" => "编辑",
 "Delete" => "删除",
 "New calendar" => "新日历",
 "Edit calendar" => "编辑日历",
 "Displayname" => "显示名称",
-"Active" => "活动",
 "Calendar color" => "日历颜色",
 "Save" => "保存",
 "Submit" => "提交",
 "Cancel" => "取消",
-"Edit an event" => "编辑事件",
-"Export" => "导出",
 "Eventinfo" => "事件信息",
 "Repeating" => "重复",
 "Alarm" => "提醒",
 "Attendees" => "参加者",
 "Share" => "共享",
 "Title of the Event" => "事件标题",
-"Category" => "分类",
-"Separate categories with commas" => "用逗号分隔分类",
-"Edit categories" => "编辑分类",
-"Access Class" => "访问分类",
+"from" => "来源",
 "All Day Event" => "全天事件",
-"From" => "自",
-"To" => "至",
 "Advanced options" => "高级选项",
 "Location" => "地点",
-"Location of the Event" => "事件地点",
+"Edit categories" => "编辑分类",
 "Description" => "描述",
-"Description of the Event" => "事件描述",
 "Repeat" => "重复",
 "Advanced" => "高级",
 "Select weekdays" => "选择星期中的某天",
@@ -187,16 +171,18 @@
 "Import" => "导入",
 "Close Dialog" => "关闭对话框",
 "Create a new event" => "创建新事件",
-"Share with:" => "共享人",
-"Shared with" => "已共享给",
 "Unshare" => "取消共享",
-"Nobody" => "没有人",
+"Send Email" => "发送邮件",
 "Shared via calendar" => "通过日历分享",
-"NOTE: Actions on events shared via calendar will affect the entire calendar sharing." => "备注：处理通过日历共享的事件将会影响整个日历共享项",
 "View an event" => "查看事件",
+"Category" => "分类",
 "No categories selected" => "无选中分类",
 "of" => "在",
+"Access Class" => "访问分类",
+"From" => "自",
 "at" => "在",
+"To" => "至",
+"Your calendars" => "您的日历",
 "General" => "常规",
 "Timezone" => "时区",
 "Update timezone automatically" => "自动更新时区",
