@@ -62,14 +62,14 @@ class OC_Calendar_Object{
 	 * in ['calendardata']
 	 */
 	public static function allInPeriod($id, $start, $end) {
-		$stmt = OCP\DB::prepare( 'SELECT * FROM `*PREFIX*clndr_objects` WHERE `calendarid` = ?'
-		.' AND ((`startdate` >= ? AND `startdate` <= ? AND `repeating` = 0)'
-		.' OR (`enddate` >= ? AND `enddate` <= ? AND `repeating` = 0)'
-		.' OR (`startdate` <= ? AND `repeating` = 1))' );
+		$stmt = OCP\DB::prepare( 'SELECT * FROM `*PREFIX*clndr_objects` WHERE `calendarid` = ? AND `objecttype`= ?' 
+		.' AND ((`startdate` >= ? AND `enddate` <= ? AND `repeating` = 0)'
+		.' OR (`enddate` >= ? AND `startdate` <= ? AND `repeating` = 0)'
+		.' OR (`startdate` <= ? AND `repeating` = 1) )' );
 		$start = self::getUTCforMDB($start);
 		$end = self::getUTCforMDB($end);
-		$result = $stmt->execute(array($id,
-					$start, $end,
+		$result = $stmt->execute(array($id,'VEVENT',
+					$start, $end,					
 					$start, $end,
 					$end));
 
@@ -153,6 +153,7 @@ class OC_Calendar_Object{
 	 * @return insertid
 	 */
 	public static function addFromDAVData($id,$uri,$data) {
+		$shared = false;
 		$calendar = OC_Calendar_Calendar::find($id);
 		if ($calendar['userid'] != OCP\User::getUser()) {
 			$sharedCalendar = OCP\Share::getItemSharedWithBySource('calendar', $id);
@@ -163,8 +164,19 @@ class OC_Calendar_Object{
 					)
 				);
 			}
+			$shared = true;
 		}
 		$object = OC_VObject::parse($data);
+		$vevent = self::getElement($object);
+
+		if($shared && isset($vevent->CLASS) && (string)$vevent->CLASS !== 'PUBLIC') {
+			throw new Sabre_DAV_Exception_PreconditionFailed(
+					OC_Calendar_App::$l10n->t(
+						'You do cannot add non-public events to a shared calendar.'
+					)
+			);
+		}
+
 		list($type,$startdate,$enddate,$summary,$repeating,$uid) = self::extractData($object);
 
 		$stmt = OCP\DB::prepare( 'INSERT INTO `*PREFIX*clndr_objects` (`calendarid`,`objecttype`,`startdate`,`enddate`,`repeating`,`summary`,`calendardata`,`uri`,`lastmodified`) VALUES(?,?,?,?,?,?,?,?,?)' );
@@ -277,8 +289,6 @@ class OC_Calendar_Object{
 
 		OCP\Util::emitHook('OC_Calendar', 'deleteEvent', $id);
 
-		OC_Calendar_App::getVCategories()->purgeObject($id);
-
 		return true;
 	}
 
@@ -502,6 +512,24 @@ class OC_Calendar_Object{
 		return $vobject;
 	}
 
+	/**
+	 * Get the contained element VEVENT, VJOURNAL, VTODO
+	 *
+	 * @param Sabre_VObject $vobject
+	 * @return Sabre_VObject|null
+	 */
+	public static function getElement($vobject) {
+		if(isset($vobject->VEVENT)) {
+			return $vobject->VEVENT;
+		}
+		elseif(isset($vobject->VJOURNAL)) {
+			return $vobject->VJOURNAL;
+		}
+		elseif(isset($vobject->VTODO)) {
+			return $vobject->VTODO;
+		}
+	}
+
 	/**
 	 * @brief Get the permissions determined by the access class of an event/todo/journal
 	 * @param Sabre_VObject $vobject Sabre VObject
@@ -509,15 +537,7 @@ class OC_Calendar_Object{
 	 * @see OCP\Share
 	 */
 	public static function getAccessClassPermissions($vobject) {
-		if(isset($vobject->VEVENT)) {
-			$velement = $vobject->VEVENT;
-		}
-		elseif(isset($vobject->VJOURNAL)) {
-			$velement = $vobject->VJOURNAL;
-		}
-		elseif(isset($vobject->VTODO)) {
-			$velement = $vobject->VTODO;
-		}
+		$velement = self::getElement($vobject);
 
 		$accessclass = $velement->getAsString('CLASS');
 
@@ -530,9 +550,9 @@ class OC_Calendar_Object{
 	 */
 	public static function getAccessClassOptions($l10n) {
 		return array(
-			'PUBLIC'       => (string)$l10n->t('Public'),
-			'PRIVATE'      => (string)$l10n->t('Private'),
-			'CONFIDENTIAL' => (string)$l10n->t('Confidential')
+			'PUBLIC'       => (string)$l10n->t('Show full event'),
+			'CONFIDENTIAL' => (string)$l10n->t('Show only busy'),
+			'PRIVATE'      => (string)$l10n->t('Hide event')
 		);
 	}
 
@@ -1036,11 +1056,9 @@ class OC_Calendar_Object{
 			$timezone = OC_Calendar_App::getTimezone();
 			$timezone = new DateTimeZone($timezone);
 			$start = new DateTime($from.' '.$fromtime, $timezone);
-			$start->setTimezone(new DateTimeZone('UTC'));
 			$end = new DateTime($to.' '.$totime, $timezone);
-			$end->setTimezone(new DateTimeZone('UTC'));
-			$vevent->setDateTime('DTSTART', $start, Sabre\VObject\Property\DateTime::UTC);
-			$vevent->setDateTime('DTEND', $end, Sabre\VObject\Property\DateTime::UTC);
+			$vevent->setDateTime('DTSTART', $start, Sabre\VObject\Property\DateTime::LOCALTZ);
+			$vevent->setDateTime('DTEND', $end, Sabre\VObject\Property\DateTime::LOCALTZ);
 		}
 		unset($vevent->DURATION);
 
@@ -1062,6 +1080,7 @@ class OC_Calendar_Object{
 	 * @return string
 	 */
 	public static function getowner($id) {
+		if ($id == 0) return null;
 		$event = self::find($id);
 		$cal = OC_Calendar_Calendar::find($event['calendarid']);
 		if($cal === false || is_array($cal) === false){
@@ -1114,8 +1133,10 @@ class OC_Calendar_Object{
 			}
 			$return['end'] = $end_dt->format('Y-m-d');
 		}else{
-			$start_dt->setTimezone(new DateTimeZone($tz));
-			$end_dt->setTimezone(new DateTimeZone($tz));
+			if($dtstart->getDateType() !== Sabre\VObject\Property\DateTime::LOCAL) {
+				$start_dt->setTimezone(new DateTimeZone($tz));
+				$end_dt->setTimezone(new DateTimeZone($tz));
+			}
 			$return['start'] = $start_dt->format('Y-m-d H:i:s');
 			$return['end'] = $end_dt->format('Y-m-d H:i:s');
 		}
