@@ -47,7 +47,8 @@ class OC_Connector_Sabre_CalDAV extends Sabre_CalDAV_Backend_Abstract {
 				'uri' => $row['uri'],
 				'principaluri' => 'principals/'.$row['userid'],
 				'{' . Sabre_CalDAV_Plugin::NS_CALENDARSERVER . '}getctag' => $row['ctag']?$row['ctag']:'0',
-				'{' . Sabre_CalDAV_Plugin::NS_CALDAV . '}supported-calendar-component-set' => new Sabre_CalDAV_Property_SupportedCalendarComponentSet($components),
+				'{' . Sabre_CalDAV_Plugin::NS_CALDAV . '}supported-calendar-component-set'
+					=> new Sabre_CalDAV_Property_SupportedCalendarComponentSet($components),
 			);
 
 			foreach($this->propertyMap as $xmlName=>$dbName) {
@@ -56,6 +57,27 @@ class OC_Connector_Sabre_CalDAV extends Sabre_CalDAV_Backend_Abstract {
 
 			$calendars[] = $calendar;
 		}
+		if(\OCP\App::isEnabled('contacts')) {
+			$ctag = 0;
+			$app = new \OCA\Contacts\App();
+			$addressBooks = $app->getAddressBooksForUser();
+			foreach($addressBooks as $addressBook) {
+				$tmp = $addressBook->lastModified();
+				if(!is_null($tmp)) {
+					$ctag = max($ctag, $tmp);
+				}
+			}
+			$ctag++;
+			$calendars[] = array(
+				'id' => 'contact_birthdays',
+				'uri' => 'contact_birthdays',
+				'{DAV:}displayname' => (string)OC_Calendar_App::$l10n->t('Contact birthdays'),
+				'principaluri' => 'principals/contact_birthdays',
+				'{' . Sabre_CalDAV_Plugin::NS_CALENDARSERVER . '}getctag' => $ctag,
+				'{' . Sabre_CalDAV_Plugin::NS_CALDAV . '}supported-calendar-component-set'
+					=> new Sabre_CalDAV_Property_SupportedCalendarComponentSet(array('VEVENT')),
+			);
+		}
 		return $calendars;
 	}
 
@@ -246,8 +268,29 @@ class OC_Connector_Sabre_CalDAV extends Sabre_CalDAV_Backend_Abstract {
 	 */
 	public function getCalendarObjects($calendarId) {
 		$data = array();
-		foreach(OC_Calendar_Object::all($calendarId) as $row) {
-			$data[] = $this->OCAddETag($row);
+		if($calendarId === 'contact_birthdays') {
+			$app = new \OCA\Contacts\App();
+			$addressBooks = $app->getAddressBooksForUser();
+			foreach($addressBooks as $addressBook) {
+				foreach($addressBook->getChildren() as $contact) {
+					$vevent = $contact->getBirthdayEvent();
+					if(is_null($vevent)) {
+						continue;
+					}
+					$data[] = $this->OCAddETag(array(
+						'id' => 0,
+						'calendarid' => 'contact_birthdays',
+						'uri' => $addressBook->getBackend()->name.'::'.$addressBook->getId().'::'.$contact->getId().'.ics',
+						'lastmodified' => $contact->lastModified(),
+						'summary' => $vevent->SUMMARY,
+						'calendardata' => $vevent->serialize()
+					));
+				}
+			}
+		} else {
+			foreach(OC_Calendar_Object::all($calendarId) as $row) {
+				$data[] = $this->OCAddETag($row);
+			}
 		}
 		return $data;
 	}
@@ -265,12 +308,34 @@ class OC_Connector_Sabre_CalDAV extends Sabre_CalDAV_Backend_Abstract {
 	 * @return array
 	 */
 	public function getCalendarObject($calendarId,$objectUri) {
+		if($calendarId === 'contact_birthdays') {
+			$objectUriArray = explode('::', $objectUri);
+			if(count($objectUriArray) === 3) {
+				$app = new \OCA\Contacts\App();
+				list($backend, $addressBookId, $contactId) = $objectUriArray;
+				$contact = $app->getContact($backend, $addressBookId, $contactId);
+				$vevent = $contact->getBirthdayEvent();
+				if(is_null($vevent)) {
+					return false;
+				}
+				return $this->OCAddETag(array(
+					'id' => 0,
+					'calendarid' => 'contact_birthdays',
+					'uri' => $contact->getBackend()->name.'::'.$contact->getParent()->getId().'::'.$contact->getId().'.ics',
+					'lastmodified' => $contact->lastModified(),
+					'calendardata' => $vevent->serialize()
+				));
+			}
+		}
 		$data = OC_Calendar_Object::findWhereDAVDataIs($calendarId,$objectUri);
 		if(is_array($data)) {
+			$data = $this->OCAddETag($data);
 			$object = OC_VObject::parse($data['calendardata']);
+			if(!$object) {
+				return false;
+			}
 			$object = OC_Calendar_Object::cleanByAccessClass($data['id'], $object);
 			$data['calendardata'] = $object->serialize();
-			$data = $this->OCAddETag($data);
 		}
 		return $data;
 	}
