@@ -6,10 +6,21 @@
  * See the COPYING-README file.
  */
 
-namespace OCA\Contacts;
+namespace OCA\Contacts\Share;
+use OCA\Contacts\App;
 
-class Share_Backend_Addressbook implements \OCP\Share_Backend_Collection {
+class Addressbook implements \OCP\Share_Backend_Collection {
 	const FORMAT_ADDRESSBOOKS = 1;
+	const FORMAT_COLLECTION = 2;
+
+	/**
+	 * @var \OCA\Contacts\App;
+	 */
+	public $app;
+
+	public function __construct() {
+		$this->app = new App(\OCP\User::getUser());
+	}
 
 	/**
 	* @brief Get the source of the item to be stored in the database
@@ -23,8 +34,11 @@ class Share_Backend_Addressbook implements \OCP\Share_Backend_Collection {
 	* The formatItems() function will translate the source returned back into the item
 	*/
 	public function isValidSource($itemSource, $uidOwner) {
-		$addressbook = Addressbook::find( $itemSource );
-		if( $addressbook === false || $addressbook['userid'] != $uidOwner) {
+		$app = new App($uidOwner);
+
+		try {
+			$app->getAddressBook('local', $itemSource);
+		} catch(\Exception $e) {
 			return false;
 		}
 		return true;
@@ -41,17 +55,25 @@ class Share_Backend_Addressbook implements \OCP\Share_Backend_Collection {
 	* If it does generate a new name e.g. name_#
 	*/
 	public function generateTarget($itemSource, $shareWith, $exclude = null) {
-		$addressbook = Addressbook::find( $itemSource );
-		$user_addressbooks = array();
-		foreach(Addressbook::all($shareWith) as $user_addressbook) {
-			$user_addressbooks[] = $user_addressbook['displayname'];
+		// Get app for the sharee
+		$app = new App($shareWith);
+		$backend = $app->getBackend('local');
+
+		// Get address book for the owner
+		$addressBook = $this->app->getBackend('local')->getAddressBook($itemSource);
+
+		$userAddressBooks = array();
+
+		foreach($backend->getAddressBooksForUser() as $userAddressBook) {
+			$userAddressBooks[] = $userAddressBook['displayname'];
 		}
-		$name = $addressbook['displayname'];
+		$name = $addressBook['displayname'] . '(' . $addressBook['owner'] . ')';
 		$suffix = '';
-		while (in_array($name.$suffix, $user_addressbooks)) {
+		while (in_array($name.$suffix, $userAddressBooks)) {
 			$suffix++;
 		}
 
+		$suffix = $suffix ? ' ' . $suffix : '';
 		return $name.$suffix;
 	}
 
@@ -68,27 +90,35 @@ class Share_Backend_Addressbook implements \OCP\Share_Backend_Collection {
 	* This function allows the backend to control the output of shared items with custom formats.
 	* It is only called through calls to the public getItem(s)Shared(With) functions.
 	*/
-	public function formatItems($items, $format, $parameters = null) {
-		$addressbooks = array();
-		if ($format == self::FORMAT_ADDRESSBOOKS) {
+	public function formatItems($items, $format, $parameters = null, $include = false) {
+		//\OCP\Util::writeLog('contacts', __METHOD__
+		//	. ' ' . $include . ' ' . print_r($items, true), \OCP\Util::DEBUG);
+		$addressBooks = array();
+		$backend = $this->app->getBackend('local');
+
+		if ($format === self::FORMAT_ADDRESSBOOKS) {
 			foreach ($items as $item) {
-				$addressbook = Addressbook::find($item['item_source']);
-				if ($addressbook) {
-					$addressbook['displayname'] = $item['item_target'];
-					$addressbook['permissions'] = $item['permissions'];
-					$addressbooks[] = $addressbook;
+				//\OCP\Util::writeLog('contacts', __METHOD__.' item owner: ' . print_r($item['uid_owner'], true) , \OCP\Util::DEBUG);
+				$addressBook = $backend->getAddressBook($item['item_source'], array('shared_by' => $item['uid_owner']));
+				if ($addressBook) {
+					$addressBook['displayname'] = $addressBook['displayname'] . ' (' . $addressBook['owner'] . ')';
+					$addressBook['permissions'] = $item['permissions'];
+					$addressBooks[] = $addressBook;
 				}
 			}
+		} elseif ($format === self::FORMAT_COLLECTION) {
+			foreach ($items as $item) {
+			}
 		}
-		return $addressbooks;
+		return $addressBooks;
 	}
 
 	public function getChildren($itemSource) {
-		$query = \OCP\DB::prepare('SELECT `id`, `fullname` FROM `*PREFIX*contacts_cards` WHERE `addressbookid` = ?');
-		$result = $query->execute(array($itemSource));
+		\OCP\Util::writeLog('contacts', __METHOD__.' item_source: ' . $itemSource, \OCP\Util::DEBUG);
+		$contacts = $this->app->getBackend('local')->getContacts($itemSource);
 		$children = array();
-		while ($contact = $result->fetchRow()) {
-			$children[] = array('source' => $contact['id'], 'target' => $contact['fullname']);
+		foreach($contacts as $contact) {
+			$children[] = array('source' => $contact['id'], 'target' => $contact['displayname']);
 		}
 		return $children;
 	}
