@@ -23,7 +23,7 @@
  *
  */
 // load needed apps
-$RUNTIME_APPTYPES=array('filesystem', 'authentication', 'logging');
+$RUNTIME_APPTYPES = array('filesystem', 'authentication', 'logging');
 
 OC_App::loadApps($RUNTIME_APPTYPES);
 
@@ -35,19 +35,24 @@ $lockBackend = new OC_Connector_Sabre_Locks();
 $requestBackend = new OC_Connector_Sabre_Request();
 
 // Create ownCloud Dir
-$publicDir = new OC_Connector_Sabre_Directory('');
+$rootDir = new OC_Connector_Sabre_Directory('');
+$objectTree = new \OC\Connector\Sabre\ObjectTree($rootDir);
 
 // Fire up server
-$server = new Sabre_DAV_Server($publicDir);
+$server = new OC_Connector_Sabre_Server($objectTree);
 $server->httpRequest = $requestBackend;
 $server->setBaseUri($baseuri);
 
 // Load plugins
-$server->addPlugin(new Sabre_DAV_Auth_Plugin($authBackend, 'ownCloud'));
+$defaults = new OC_Defaults();
+$server->addPlugin(new Sabre_DAV_Auth_Plugin($authBackend, $defaults->getName()));
 $server->addPlugin(new Sabre_DAV_Locks_Plugin($lockBackend));
 $server->addPlugin(new Sabre_DAV_Browser_Plugin(false)); // Show something in the Browser, but no upload
+$server->addPlugin(new OC_Connector_Sabre_FilesPlugin());
+$server->addPlugin(new OC_Connector_Sabre_AbortedUploadDetectionPlugin());
 $server->addPlugin(new OC_Connector_Sabre_QuotaPlugin());
 $server->addPlugin(new OC_Connector_Sabre_MaintenancePlugin());
+$server->addPlugin(new OC_Connector_Sabre_ExceptionLoggerPlugin('webdav'));
 
 // And off we go!
 $server->exec();
