@@ -21,29 +21,28 @@
 *
 */
 
-// Init owncloud
- 
-
 // Check if we are a user
 OCP\User::checkLoggedIn();
 
+// don't block php session during download
+session_write_close();
+
 $filename = $_GET["file"];
 
-if(!OC_Filesystem::file_exists($filename)){
+if(!\OC\Files\Filesystem::file_exists($filename)) {
 	header("HTTP/1.0 404 Not Found");
 	$tmpl = new OCP\Template( '', '404', 'guest' );
-	$tmpl->assign('file',$filename);
+	$tmpl->assign('file', $filename);
 	$tmpl->printPage();
 	exit;
 }
 
-$ftype=OC_Filesystem::getMimeType( $filename );
+$ftype=\OC\Files\Filesystem::getMimeType( $filename );
 
 header('Content-Type:'.$ftype);
-header('Content-Disposition: attachment; filename="'.basename($filename).'"');
+OCP\Response::setContentDispositionHeader(basename($filename), 'attachment');
 OCP\Response::disableCaching();
-header('Content-Length: '.OC_Filesystem::filesize($filename));
+header('Content-Length: '.\OC\Files\Filesystem::filesize($filename));
 
-@ob_end_clean();
-OC_Filesystem::readfile( $filename );
-?>
+OC_Util::obEnd();
+\OC\Files\Filesystem::readfile( $filename );
