@@ -1,33 +1,25 @@
-<?php $TRANSLATIONS = array(
+<?php
+$TRANSLATIONS = array(
 "There is no error, the file uploaded with success" => "Keen Feeler, Datei ass komplett ropgelueden ginn",
-"The uploaded file exceeds the upload_max_filesize directive in php.ini" => "Déi ropgelueden Datei ass méi grouss wei d'upload_max_filesize Eegenschaft an der php.ini",
 "The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form" => "Déi ropgelueden Datei ass méi grouss wei d'MAX_FILE_SIZE Eegenschaft déi an der HTML form uginn ass",
 "The uploaded file was only partially uploaded" => "Déi ropgelueden Datei ass nëmmen hallef ropgelueden ginn",
-"No file was uploaded" => "Et ass keng Datei ropgelueden ginn",
+"No file was uploaded" => "Et ass kee Fichier ropgeluede ginn",
 "Missing a temporary folder" => "Et feelt en temporären Dossier",
 "Failed to write to disk" => "Konnt net op den Disk schreiwen",
 "Files" => "Dateien",
-"Delete" => "Läschen",
-"already exists" => "existéiert schonn",
-"replace" => "ersetzen",
-"cancel" => "ofbriechen",
-"replaced" => "ersat",
-"undo" => "réckgängeg man",
-"with" => "mat",
-"deleted" => "geläscht",
-"generating ZIP-file, it may take some time." => "Et  gëtt eng ZIP-File generéiert, dëst ka bëssen daueren.",
-"Unable to upload your file as it is a directory or has 0 bytes" => "Kann deng Datei net eroplueden well et en Dossier ass oder 0 byte grouss ass.",
-"Upload Error" => "Fehler beim eroplueden",
 "Upload cancelled." => "Upload ofgebrach.",
 "File upload is in progress. Leaving the page now will cancel the upload." => "File Upload am gaang. Wann's de des Säit verléiss gëtt den Upload ofgebrach.",
-"Invalid name, '/' is not allowed." => "Ongültege Numm, '/' net erlaabt.",
+"Share" => "Deelen",
+"Rename" => "Ëm-benennen",
+"undo" => "réckgängeg man",
+"_%n folder_::_%n folders_" => array("",""),
+"_%n file_::_%n files_" => array("",""),
+"_Uploading %n file_::_Uploading %n files_" => array("",""),
+"Error" => "Fehler",
 "Name" => "Numm",
 "Size" => "Gréisst",
 "Modified" => "Geännert",
-"folder" => "Dossier",
-"folders" => "Dossieren",
-"file" => "Datei",
-"files" => "Dateien",
+"Upload" => "Eroplueden",
 "File handling" => "Fichier handling",
 "Maximum upload size" => "Maximum Upload Gréisst ",
 "max. possible: " => "max. méiglech:",
@@ -35,17 +27,18 @@
 "Enable ZIP-download" => "ZIP-download erlaben",
 "0 is unlimited" => "0 ass onlimitéiert",
 "Maximum input size for ZIP files" => "Maximal Gréisst fir ZIP Fichieren",
+"Save" => "Späicheren",
 "New" => "Nei",
 "Text file" => "Text Fichier",
 "Folder" => "Dossier",
-"From url" => "From URL",
-"Upload" => "Eroplueden",
 "Cancel upload" => "Upload ofbriechen",
 "Nothing in here. Upload something!" => "Hei ass näischt. Lued eppes rop!",
-"Share" => "Share",
-"Download" => "Eroflueden",
+"Download" => "Download",
+"Unshare" => "Net méi deelen",
+"Delete" => "Läschen",
 "Upload too large" => "Upload ze grouss",
 "The files you are trying to upload exceed the maximum size for file uploads on this server." => "Déi Dateien déi Dir probéiert erop ze lueden sinn méi grouss wei déi Maximal Gréisst déi op dësem Server erlaabt ass.",
 "Files are being scanned, please wait." => "Fichieren gi gescannt, war weg.",
 "Current scanning" => "Momentane Scan"
 );
+$PLURAL_FORMS = "nplurals=2; plural=(n != 1);";
