@@ -23,6 +23,8 @@ OC_JSON::checkLoggedIn();
 OCP\JSON::callCheck();
 OC_App::loadApps();
 
+$defaults = new \OCP\Defaults();
+
 if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSource'])) {
 	switch ($_POST['action']) {
 		case 'share':
@@ -33,15 +35,16 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 					if ($shareType === OCP\Share::SHARE_TYPE_LINK && $shareWith == '') {
 						$shareWith = null;
 					}
-					
+
 					$token = OCP\Share::shareItem(
 						$_POST['itemType'],
 						$_POST['itemSource'],
 						$shareType,
 						$shareWith,
-						$_POST['permissions']
+						$_POST['permissions'],
+						$_POST['itemSourceName']
 					);
-					
+
 					if (is_string($token)) {
 						OC_JSON::success(array('data' => array('token' => $token)));
 					} else {
@@ -81,7 +84,111 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 				($return) ? OC_JSON::success() : OC_JSON::error();
 			}
 			break;
+		case 'informRecipients':
+
+			$l = OC_L10N::get('core');
+
+			$shareType = (int) $_POST['shareType'];
+			$itemType = $_POST['itemType'];
+			$itemSource = $_POST['itemSource'];
+			$recipient = $_POST['recipient'];
+			$ownerDisplayName = \OCP\User::getDisplayName();
+			$from = \OCP\Util::getDefaultEmailAddress('sharing-noreply');
+
+			$noMail = array();
+			$recipientList = array();
+
+			if($shareType === \OCP\Share::SHARE_TYPE_USER) {
+				$recipientList[] = $recipient;
+			} elseif ($shareType === \OCP\Share::SHARE_TYPE_GROUP) {
+				$recipientList = \OC_Group::usersInGroup($recipient);
+			}
+
+			// don't send a mail to the user who shared the file
+			$recipientList = array_diff($recipientList, array(\OCP\User::getUser()));
+
+			// send mail to all recipients with an email address
+			foreach ($recipientList as $recipient) {
+				//get correct target folder name
+				$email = OC_Preferences::getValue($recipient, 'settings', 'email', '');
+
+				if ($email !== '') {
+					$displayName = \OCP\User::getDisplayName($recipient);
+					$items = \OCP\Share::getItemSharedWithUser($itemType, $itemSource, $recipient);
+					$filename = trim($items[0]['file_target'], '/');
+					$subject = (string)$l->t('%s shared »%s« with you', array($ownerDisplayName, $filename));
+					$expiration = null;
+					if (isset($items[0]['expiration'])) {
+						try {
+							$date = new DateTime($items[0]['expiration']);
+							$expiration = $l->l('date', $date->getTimestamp());
+						} catch (Exception $e) {
+							\OCP\Util::writeLog('sharing', "Couldn't read date: " . $e->getMessage(), \OCP\Util::ERROR);
+						}
+					}
+
+					if ($itemType === 'folder') {
+						$foldername = "/Shared/" . $filename;
+					} else {
+						// if it is a file we can just link to the Shared folder,
+						// that's the place where the user will find the file
+						$foldername = "/Shared";
+					}
+
+					$link = \OCP\Util::linkToAbsolute('files', 'index.php', array("dir" => $foldername));
+
+					$content = new OC_Template("core", "mail", "");
+					$content->assign('link', $link);
+					$content->assign('user_displayname', $ownerDisplayName);
+					$content->assign('filename', $filename);
+					$content->assign('expiration', $expiration);
+					$text = $content->fetchPage();
+
+					$content = new OC_Template("core", "altmail", "");
+					$content->assign('link', $link);
+					$content->assign('user_displayname', $ownerDisplayName);
+					$content->assign('filename', $filename);
+					$content->assign('expiration', $expiration);
+					$alttext = $content->fetchPage();
+
+					$default_from = OCP\Util::getDefaultEmailAddress('sharing-noreply');
+					$from = OCP\Config::getUserValue(\OCP\User::getUser(), 'settings', 'email', $default_from);
+
+					// send it out now
+					try {
+						OCP\Util::sendMail($email, $displayName, $subject, $text, $from, $ownerDisplayName, 1, $alttext);
+					} catch (Exception $exception) {
+						$noMail[] = \OCP\User::getDisplayName($recipient);
+					}
+				}
+			}
+
+			\OCP\Share::setSendMailStatus($itemType, $itemSource, $shareType, true);
+
+			if (empty($noMail)) {
+				OCP\JSON::success();
+			} else {
+				OCP\JSON::error(array(
+					'data' => array(
+						'message' => $l->t("Couldn't send mail to following users: %s ",
+								implode(', ', $noMail)
+								)
+						)
+					));
+			}
+			break;
+		case 'informRecipientsDisabled':
+			$itemSource = $_POST['itemSource'];
+			$shareType = $_POST['shareType'];
+			$itemType = $_POST['itemType'];
+			$recipient = $_POST['recipient'];
+			\OCP\Share::setSendMailStatus($itemType, $itemSource, $shareType, false);
+			OCP\JSON::success();
+			break;
+
 		case 'email':
+			// enable l10n support
+			$l = OC_L10N::get('core');
 			// read post variables
 			$user = OCP\USER::getUser();
 			$displayName = OCP\User::getDisplayName();
@@ -90,31 +197,61 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 			$file = $_POST['file'];
 			$to_address = $_POST['toaddress'];
 
-			// enable l10n support
-			$l = OC_L10N::get('core');
+			$expiration = null;
+			if (isset($_POST['expiration']) && $_POST['expiration'] !== '') {
+				try {
+					$date = new DateTime($_POST['expiration']);
+					$expiration = $l->l('date', $date->getTimestamp());
+				} catch (Exception $e) {
+					\OCP\Util::writeLog('sharing', "Couldn't read date: " . $e->getMessage(), \OCP\Util::ERROR);
+				}
+
+			}
 
 			// setup the email
-			$subject = (string)$l->t('User %s shared a file with you', $displayName);
-			if ($type === 'folder')
-				$subject = (string)$l->t('User %s shared a folder with you', $displayName);
+			$subject = (string)$l->t('%s shared »%s« with you', array($displayName, $file));
 
-			$text = (string)$l->t('User %s shared the file "%s" with you. It is available for download here: %s',
-				array($displayName, $file, $link));
-			if ($type === 'folder')
-				$text = (string)$l->t('User %s shared the folder "%s" with you. It is available for download here: %s',
-					array($displayName, $file, $link));
+			$content = new OC_Template("core", "mail", "");
+			$content->assign ('link', $link);
+			$content->assign ('type', $type);
+			$content->assign ('user_displayname', $displayName);
+			$content->assign ('filename', $file);
+			$content->assign('expiration', $expiration);
+			$text = $content->fetchPage();
 
+			$content = new OC_Template("core", "altmail", "");
+			$content->assign ('link', $link);
+			$content->assign ('type', $type);
+			$content->assign ('user_displayname', $displayName);
+			$content->assign ('filename', $file);
+			$content->assign('expiration', $expiration);
+			$alttext = $content->fetchPage();
 
 			$default_from = OCP\Util::getDefaultEmailAddress('sharing-noreply');
 			$from_address = OCP\Config::getUserValue($user, 'settings', 'email', $default_from );
 
 			// send it out now
-			try {
-				OCP\Util::sendMail($to_address, $to_address, $subject, $text, $from_address, $displayName);
-				OCP\JSON::success();
-			} catch (Exception $exception) {
-				OCP\JSON::error(array('data' => array('message' => OC_Util::sanitizeHTML($exception->getMessage()))));
+			$rs = explode(' ', $to_address);
+			$failed = array();
+			foreach ($rs as $r) {
+				try {
+					\OCP\Util::sendMail($r, $r, $subject, $text, $from_address, $displayName, 1, $alttext);
+				} catch (\Exception $e) {
+					\OCP\Util::writeLog('sharing', "Can't send mail with public link to $r: " . $e->getMessage(), \OCP\Util::ERROR);
+					$failed[] = $r;
+				}
 			}
+
+			if (empty($failed)) {
+				OCP\JSON::success();
+			} else {
+				OCP\JSON::error(array(
+					'data' => array(
+						'message' => $l->t("Couldn't send mail to following users: %s ",
+								implode(', ', $failed)
+							)
+					)
+				));			}
 			break;
 	}
 } else if (isset($_GET['fetch'])) {
@@ -176,10 +313,10 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 // 						}
 // 					}
 // 				}
+				$groups = OC_Group::getGroups($_GET['search']);
 				if ($sharePolicy == 'groups_only') {
-					$groups = OC_Group::getUserGroups(OC_User::getUser());
-				} else {
-					$groups = OC_Group::getGroups();
+					$usergroups = OC_Group::getUserGroups(OC_User::getUser());
+					$groups = array_intersect($groups, $usergroups);
 				}
 				$count = 0;
 				$users = array();
@@ -188,7 +325,7 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 				while ($count < 15 && count($users) == $limit) {
 					$limit = 15 - $count;
 					if ($sharePolicy == 'groups_only') {
-						$users = OC_Group::DisplayNamesInGroups($groups, $_GET['search'], $limit, $offset);
+						$users = OC_Group::DisplayNamesInGroups($usergroups, $_GET['search'], $limit, $offset);
 					} else {
 						$users = OC_User::getDisplayNames($_GET['search'], $limit, $offset);
 					}
@@ -200,23 +337,27 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 							&& $uid != OC_User::getUser()) {
 							$shareWith[] = array(
 								'label' => $displayName,
-								'value' => array('shareType' => OCP\Share::SHARE_TYPE_USER,
-								'shareWith' => $uid)
+								'value' => array(
+									'shareType' => OCP\Share::SHARE_TYPE_USER,
+									'shareWith' => $uid)
 							);
 							$count++;
 						}
 					}
 				}
 				$count = 0;
+
+				// enable l10n support
+				$l = OC_L10N::get('core');
+
 				foreach ($groups as $group) {
 					if ($count < 15) {
-						if (stripos($group, $_GET['search']) !== false
-							&& (!isset($_GET['itemShares'])
+						if (!isset($_GET['itemShares'])
 							|| !isset($_GET['itemShares'][OCP\Share::SHARE_TYPE_GROUP])
 							|| !is_array($_GET['itemShares'][OCP\Share::SHARE_TYPE_GROUP])
-							|| !in_array($group, $_GET['itemShares'][OCP\Share::SHARE_TYPE_GROUP]))) {
+							|| !in_array($group, $_GET['itemShares'][OCP\Share::SHARE_TYPE_GROUP])) {
 							$shareWith[] = array(
-								'label' => $group.' (group)',
+								'label' => $group,
 								'value' => array(
 									'shareType' => OCP\Share::SHARE_TYPE_GROUP,
 									'shareWith' => $group
@@ -228,6 +369,10 @@ if (isset($_POST['action']) && isset($_POST['itemType']) && isset($_POST['itemSo
 						break;
 					}
 				}
+				$sorter = new \OC\Share\SearchResultSorter($_GET['search'],
+														   'label',
+														   new \OC\Log());
+				usort($shareWith, array($sorter, 'sort'));
 				OC_JSON::success(array('data' => $shareWith));
 			}
 			break;
