@@ -1,6 +1,6 @@
 /**
  * Disable console output unless DEBUG mode is enabled.
- * Add 
+ * Add
  *	 define('DEBUG', true);
  * To the end of config/config.php to enable debug mode.
  * The undefined checks fix the broken ie8 console
@@ -11,8 +11,17 @@ var oc_webroot;
 var oc_current_user = document.getElementsByTagName('head')[0].getAttribute('data-user');
 var oc_requesttoken = document.getElementsByTagName('head')[0].getAttribute('data-requesttoken');
 
+window.oc_config = window.oc_config || {};
+
 if (typeof oc_webroot === "undefined") {
-	oc_webroot = location.pathname.substr(0, location.pathname.lastIndexOf('/'));
+	oc_webroot = location.pathname;
+	var pos = oc_webroot.indexOf('/index.php/');
+	if (pos !== -1) {
+		oc_webroot = oc_webroot.substr(0, pos);
+	}
+	else {
+		oc_webroot = oc_webroot.substr(0, oc_webroot.lastIndexOf('/'));
+	}
 }
 if (oc_debug !== true || typeof console === "undefined" || typeof console.log === "undefined") {
 	if (!window.console) {
@@ -24,60 +33,125 @@ if (oc_debug !== true || typeof console === "undefined" || typeof console.log ==
 	}
 }
 
-/**
- * translate a string
- * @param app the id of the app for which to translate the string
- * @param text the string to translate
- * @return string
- */
-function t(app,text, vars){
-	if( !( t.cache[app] )){
-		$.ajax(OC.filePath('core','ajax','translations.php'),{
-			async:false,//todo a proper sollution for this without sync ajax calls
-			data:{'app': app},
-			type:'POST',
-			success:function(jsondata){
+function initL10N(app) {
+	if (!( t.cache[app] )) {
+		$.ajax(OC.filePath('core', 'ajax', 'translations.php'), {
+			async: false,//todo a proper solution for this without sync ajax calls
+			data: {'app': app},
+			type: 'POST',
+			success: function (jsondata) {
 				t.cache[app] = jsondata.data;
+				t.plural_form = jsondata.plural_form;
 			}
 		});
 
 		// Bad answer ...
-		if( !( t.cache[app] )){
+		if (!( t.cache[app] )) {
 			t.cache[app] = [];
 		}
 	}
-	var _build = function (text, vars) {
-		return text.replace(/{([^{}]*)}/g,
+	if (typeof t.plural_function[app] == 'undefined') {
+		t.plural_function[app] = function (n) {
+			var p = (n != 1) ? 1 : 0;
+			return { 'nplural' : 2, 'plural' : p };
+		};
+
+		/**
+		 * code below has been taken from jsgettext - which is LGPL licensed
+		 * https://developer.berlios.de/projects/jsgettext/
+		 * http://cvs.berlios.de/cgi-bin/viewcvs.cgi/jsgettext/jsgettext/lib/Gettext.js
+		 */
+		var pf_re = new RegExp('^(\\s*nplurals\\s*=\\s*[0-9]+\\s*;\\s*plural\\s*=\\s*(?:\\s|[-\\?\\|&=!<>+*/%:;a-zA-Z0-9_\(\)])+)', 'm');
+		if (pf_re.test(t.plural_form)) {
+			//ex english: "Plural-Forms: nplurals=2; plural=(n != 1);\n"
+			//pf = "nplurals=2; plural=(n != 1);";
+			//ex russian: nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10< =4 && (n%100<10 or n%100>=20) ? 1 : 2)
+			//pf = "nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 || n%100>=20) ? 1 : 2)";
+			var pf = t.plural_form;
+			if (! /;\s*$/.test(pf)) pf = pf.concat(';');
+			/* We used to use eval, but it seems IE has issues with it.
+			 * We now use "new Function", though it carries a slightly
+			 * bigger performance hit.
+			 var code = 'function (n) { var plural; var nplurals; '+pf+' return { "nplural" : nplurals, "plural" : (plural === true ? 1 : plural ? plural : 0) }; };';
+			 Gettext._locale_data[domain].head.plural_func = eval("("+code+")");
+			 */
+			var code = 'var plural; var nplurals; '+pf+' return { "nplural" : nplurals, "plural" : (plural === true ? 1 : plural ? plural : 0) };';
+			t.plural_function[app] = new Function("n", code);
+		} else {
+			console.log("Syntax error in language file. Plural-Forms header is invalid ["+t.plural_forms+"]");
+		}
+	}
+}
+/**
+ * translate a string
+ * @param app the id of the app for which to translate the string
+ * @param text the string to translate
+ * @param vars (optional) FIXME
+ * @param count (optional) number to replace %n with
+ * @return string
+ */
+function t(app, text, vars, count){
+	initL10N(app);
+	var _build = function (text, vars, count) {
+		return text.replace(/%n/g, count).replace(/{([^{}]*)}/g,
 			function (a, b) {
 				var r = vars[b];
 				return typeof r === 'string' || typeof r === 'number' ? r : a;
 			}
 		);
 	};
+	var translation = text;
 	if( typeof( t.cache[app][text] ) !== 'undefined' ){
-		if(typeof vars === 'object') {
-			return _build(t.cache[app][text], vars);
-		} else {
-			return t.cache[app][text];
-		}
+		translation = t.cache[app][text];
 	}
-	else{
-		if(typeof vars === 'object') {
-			return _build(text, vars);
-		} else {
-			return text;
-		}
+
+	if(typeof vars === 'object' || count !== undefined ) {
+		return _build(translation, vars, count);
+	} else {
+		return translation;
 	}
 }
-t.cache={};
+t.cache = {};
+// different apps might or might not redefine the nplurals function correctly
+// this is to make sure that a "broken" app doesn't mess up with the
+// other app's plural function
+t.plural_function = {};
 
-/*
+/**
+ * translate a string
+ * @param app the id of the app for which to translate the string
+ * @param text_singular the string to translate for exactly one object
+ * @param text_plural the string to translate for n objects
+ * @param count number to determine whether to use singular or plural
+ * @param vars (optional) FIXME
+ * @return string
+ */
+function n(app, text_singular, text_plural, count, vars) {
+	initL10N(app);
+	var identifier = '_' + text_singular + '_::_' + text_plural + '_';
+	if( typeof( t.cache[app][identifier] ) !== 'undefined' ){
+		var translation = t.cache[app][identifier];
+		if ($.isArray(translation)) {
+			var plural = t.plural_function[app](count);
+			return t(app, translation[plural.plural], vars, count);
+		}
+	}
+
+	if(count === 1) {
+		return t(app, text_singular, vars, count);
+	}
+	else{
+		return t(app, text_plural, vars, count);
+	}
+}
+
+/**
 * Sanitizes a HTML string
-* @param string
+* @param s string
 * @return Sanitized string
 */
 function escapeHTML(s) {
-		return s.toString().split('&').join('&amp;').split('<').join('&lt;').split('"').join('&quot;');
+	return s.toString().split('&').join('&amp;').split('<').join('&lt;').split('"').join('&quot;');
 }
 
 /**
@@ -96,10 +170,35 @@ var OC={
 	PERMISSION_UPDATE:2,
 	PERMISSION_DELETE:8,
 	PERMISSION_SHARE:16,
+	PERMISSION_ALL:31,
 	webroot:oc_webroot,
 	appswebroots:(typeof oc_appswebroots !== 'undefined') ? oc_appswebroots:false,
 	currentUser:(typeof oc_current_user!=='undefined')?oc_current_user:false,
 	coreApps:['', 'admin','log','search','settings','core','3rdparty'],
+
+	/**
+	 * Generates the absolute url for the given relative url, which can contain parameters.
+	 *
+	 * @returns {string}
+	 * @param {string} url
+	 * @param params
+	 */
+	generateUrl: function(url, params) {
+		var _build = function (text, vars) {
+			return text.replace(/{([^{}]*)}/g,
+				function (a, b) {
+					var r = vars[b];
+					return typeof r === 'string' || typeof r === 'number' ? r : a;
+				}
+			);
+		};
+		if (url.charAt(0) !== '/') {
+			url = '/' + url;
+
+		}
+		return OC.webroot + '/index.php' + _build(url, params);
+	},
+
 	/**
 	 * get an absolute url to a file in an appen
 	 * @param app the id of the app the file belongs to
@@ -259,6 +358,38 @@ var OC={
 		var date = new Date(1000*mtime);
 		return date.getDate()+'.'+(date.getMonth()+1)+'.'+date.getFullYear()+', '+date.getHours()+':'+date.getMinutes();
 	},
+	/**
+	 * Parses a URL query string into a JS map
+	 * @param queryString query string in the format param1=1234&param2=abcde&param3=xyz
+	 * @return map containing key/values matching the URL parameters
+	 */
+	parseQueryString:function(queryString){
+		var parts,
+			components,
+			result = {},
+			key,
+			value;
+		if (!queryString){
+			return null;
+		}
+		if (queryString[0] === '?'){
+			queryString = queryString.substr(1);
+		}
+		parts = queryString.split('&');
+		for (var i = 0; i < parts.length; i++){
+			components = parts[i].split('=');
+			if (!components.length){
+				continue;
+			}
+			key = decodeURIComponent(components[0]);
+			if (!key){
+				continue;
+			}
+			value = components[1];
+			result[key] = value && decodeURIComponent(value);
+		}
+		return result;
+	},
 	/**
 	 * Opens a popup with the setting for an app.
 	 * @param appid String. The ID of the app e.g. 'calendar', 'contacts' or 'files'.
@@ -356,10 +487,10 @@ OC.Notification={
 	},
 	show: function(text) {
 		if(($('#notification').filter('span.undo').length == 1) || OC.Notification.isHidden()){
-			$('#notification').html(text);
+			$('#notification').text(text);
 			$('#notification').fadeIn().css("display","inline");
 		}else{
-			OC.Notification.queuedNotifications.push($(text).html());
+			OC.Notification.queuedNotifications.push($('<div/>').text(text).html());
 		}
 	},
 	isHidden: function() {
@@ -369,11 +500,58 @@ OC.Notification={
 
 OC.Breadcrumb={
 	container:null,
-	crumbs:[],
-	push:function(name, link){
-		if(!OC.Breadcrumb.container){//default
-			OC.Breadcrumb.container=$('#controls');
+	show:function(dir, leafname, leaflink){
+		if(!this.container){//default
+			this.container=$('#controls');
 		}
+		this._show(this.container, dir, leafname, leaflink);
+	},
+	_show:function(container, dir, leafname, leaflink){
+		var self = this;
+
+		this._clear(container);
+
+		// show home + path in subdirectories
+		if (dir) {
+			//add home
+			var link = OC.linkTo('files','index.php');
+
+			var crumb=$('<div/>');
+			crumb.addClass('crumb');
+
+			var crumbLink=$('<a/>');
+			crumbLink.attr('href',link);
+
+			var crumbImg=$('<img/>');
+			crumbImg.attr('src',OC.imagePath('core','places/home'));
+			crumbLink.append(crumbImg);
+			crumb.append(crumbLink);
+			container.prepend(crumb);
+
+			//add path parts
+			var segments = dir.split('/');
+			var pathurl = '';
+			jQuery.each(segments, function(i,name) {
+				if (name !== '') {
+					pathurl = pathurl+'/'+name;
+					var link = OC.linkTo('files','index.php')+'?dir='+encodeURIComponent(pathurl);
+					self._push(container, name, link);
+				}
+			});
+		}
+
+		//add leafname
+		if (leafname && leaflink) {
+			this._push(container, leafname, leaflink);
+		}
+	},
+	push:function(name, link){
+		if(!this.container){//default
+			this.container=$('#controls');
+		}
+		return this._push(OC.Breadcrumb.container, name, link);
+	},
+	_push:function(container, name, link){
 		var crumb=$('<div/>');
 		crumb.addClass('crumb').addClass('last');
 
@@ -382,30 +560,30 @@ OC.Breadcrumb={
 		crumbLink.text(name);
 		crumb.append(crumbLink);
 
-		var existing=OC.Breadcrumb.container.find('div.crumb');
+		var existing=container.find('div.crumb');
 		if(existing.length){
 			existing.removeClass('last');
 			existing.last().after(crumb);
 		}else{
-			OC.Breadcrumb.container.append(crumb);
+			container.prepend(crumb);
 		}
-		OC.Breadcrumb.crumbs.push(crumb);
 		return crumb;
 	},
 	pop:function(){
-		if(!OC.Breadcrumb.container){//default
-			OC.Breadcrumb.container=$('#controls');
+		if(!this.container){//default
+			this.container=$('#controls');
 		}
-		OC.Breadcrumb.container.find('div.crumb').last().remove();
-		OC.Breadcrumb.container.find('div.crumb').last().addClass('last');
-		OC.Breadcrumb.crumbs.pop();
+		this.container.find('div.crumb').last().remove();
+		this.container.find('div.crumb').last().addClass('last');
 	},
 	clear:function(){
-		if(!OC.Breadcrumb.container){//default
-			OC.Breadcrumb.container=$('#controls');
+		if(!this.container){//default
+			this.container=$('#controls');
 		}
-		OC.Breadcrumb.container.find('div.crumb').remove();
-		OC.Breadcrumb.crumbs=[];
+		this._clear(this.container);
+	},
+	_clear:function(container) {
+		container.find('div.crumb').remove();
 	}
 };
 
@@ -446,52 +624,6 @@ if(typeof localStorage !=='undefined' && localStorage !== null){
 	};
 }
 
-/**
- * implement Array.filter for browsers without native support
- */
-if (!Array.prototype.filter) {
-	Array.prototype.filter = function(fun /*, thisp*/) {
-		var len = this.length >>> 0;
-		if (typeof fun !== "function"){
-			throw new TypeError();
-		}
-
-		var res = [];
-		var thisp = arguments[1];
-		for (var i = 0; i < len; i++) {
-			if (i in this) {
-				var val = this[i]; // in case fun mutates this
-				if (fun.call(thisp, val, i, this))
-					res.push(val);
-			}
-		}
-		return res;
-	};
-}
-/**
- * implement Array.indexOf for browsers without native support
- */
-if (!Array.prototype.indexOf){
-	Array.prototype.indexOf = function(elt /*, from*/)
-	{
-		var len = this.length;
-
-		var from = Number(arguments[1]) || 0;
-		from = (from < 0) ? Math.ceil(from) : Math.floor(from);
-		if (from < 0){
-			from += len;
-		}
-
-		for (; from < len; from++)
-		{
-			if (from in this && this[from] === elt){
-				return from;
-			}
-		}
-		return -1;
-	};
-}
-
 /**
  * check if the browser support svg images
  */
@@ -599,8 +731,44 @@ function fillWindow(selector) {
 	console.warn("This function is deprecated! Use CSS instead");
 }
 
-$(document).ready(function(){
-	sessionHeartBeat();
+/**
+ * Initializes core
+ */
+function initCore() {
+
+	/**
+	 * Calls the server periodically to ensure that session doesnt
+	 * time out
+	 */
+	function initSessionHeartBeat(){
+		// max interval in seconds set to 24 hours
+		var maxInterval = 24 * 3600;
+		// interval in seconds
+		var interval = 900;
+		if (oc_config.session_lifetime) {
+			interval = Math.floor(oc_config.session_lifetime / 2);
+		}
+		// minimum one minute
+		if (interval < 60) {
+			interval = 60;
+		}
+		if (interval > maxInterval) {
+			interval = maxInterval;
+		}
+		OC.Router.registerLoadedCallback(function(){
+			var url = OC.Router.generate('heartbeat');
+			setInterval(function(){
+				$.post(url);
+			}, interval * 1000);
+		});
+	}
+
+	// session heartbeat (defalts to enabled)
+	if (typeof(oc_config.session_keepalive) === 'undefined' ||
+		!!oc_config.session_keepalive) {
+
+		initSessionHeartBeat();
+	}
 
 	if(!SVGSupport()){ //replace all svg images with png images for browser that dont support svg
 		replaceSVG();
@@ -628,11 +796,17 @@ $(document).ready(function(){
 			}
 		}else if(event.keyCode===27){//esc
 			OC.search.hide();
+			if (FileList && typeof FileList.unfilter === 'function') { //TODO add hook system
+				FileList.unfilter();
+			}
 		}else{
 			var query=$('#searchbox').val();
 			if(OC.search.lastQuery!==query){
 				OC.search.lastQuery=query;
 				OC.search.currentResult=-1;
+				if (FileList && typeof FileList.filter === 'function') { //TODO add hook system
+						FileList.filter(query);
+				}
 				if(query.length>2){
 					OC.search(query);
 				}else{
@@ -644,13 +818,17 @@ $(document).ready(function(){
 		}
 	});
 
-	// 'show password' checkbox
-	$('#password').showPassword();
-	$('#adminpass').showPassword();	
-	$('#pass2').showPassword();
+	var setShowPassword = function(input, label) {
+		input.showPassword().keyup();
+	};
+	setShowPassword($('#adminpass'), $('label[for=show]'));
+	setShowPassword($('#pass2'), $('label[for=personal-show]'));
+	setShowPassword($('#dbpass'), $('label[for=dbpassword]'));
 
 	//use infield labels
-	$("label.infield").inFieldLabels();
+	$("label.infield").inFieldLabels({
+		pollDuration: 100
+	});
 
 	var checkShowCredentials = function() {
 		var empty = false;
@@ -691,50 +869,22 @@ $(document).ready(function(){
 	});
 
 	// all the tipsy stuff needs to be here (in reverse order) to work
-	$('.jp-controls .jp-previous').tipsy({gravity:'nw', fade:true, live:true});
-	$('.jp-controls .jp-next').tipsy({gravity:'n', fade:true, live:true});
 	$('.displayName .action').tipsy({gravity:'se', fade:true, live:true});
 	$('.password .action').tipsy({gravity:'se', fade:true, live:true});
 	$('#upload').tipsy({gravity:'w', fade:true});
 	$('.selectedActions a').tipsy({gravity:'s', fade:true, live:true});
-	$('a.delete').tipsy({gravity: 'e', fade:true, live:true});
+	$('a.action.delete').tipsy({gravity:'e', fade:true, live:true});
 	$('a.action').tipsy({gravity:'s', fade:true, live:true});
-	$('#headerSize').tipsy({gravity:'s', fade:true, live:true});
-	$('td.filesize').tipsy({gravity:'s', fade:true, live:true});
 	$('td .modified').tipsy({gravity:'s', fade:true, live:true});
 
 	$('input').tipsy({gravity:'w', fade:true});
 	$('input[type=text]').focus(function(){
 		this.select();
 	});
-});
-
-if (!Array.prototype.map){
-	Array.prototype.map = function(fun /*, thisp */){
-		"use strict";
-
-		if (this === void 0 || this === null){
-			throw new TypeError();
-		}
-
-		var t = Object(this);
-		var len = t.length >>> 0;
-		if (typeof fun !== "function"){
-			throw new TypeError();
-		}
-
-		var res = new Array(len);
-		var thisp = arguments[1];
-		for (var i = 0; i < len; i++){
-			if (i in t){
-				res[i] = fun.call(thisp, t[i], i, t);
-			}
-		}
-
-		return res;
-	};
 }
 
+$(document).ready(initCore);
+
 /**
  * Filter Jquery selector by attribute value
  */
@@ -756,14 +906,6 @@ function humanFileSize(size) {
 	return relativeSize + ' ' + readableFormat;
 }
 
-function simpleFileSize(bytes) {
-	var mbytes = Math.round(bytes/(1024*1024/10))/10;
-	if(bytes == 0) { return '0'; }
-	else if(mbytes < 0.1) { return '< 0.1'; }
-	else if(mbytes > 1000) { return '> 1000'; }
-	else { return mbytes.toFixed(1); }
-}
-
 function formatDate(date){
 	if(typeof date=='number'){
 		date=new Date(date);
@@ -771,6 +913,13 @@ function formatDate(date){
 	return $.datepicker.formatDate(datepickerFormatDate, date)+' '+date.getHours()+':'+((date.getMinutes()<10)?'0':'')+date.getMinutes();
 }
 
+// taken from http://stackoverflow.com/questions/1403888/get-url-parameter-with-jquery
+function getURLParameter(name) {
+	return decodeURI(
+			(RegExp(name + '=' + '(.+?)(&|$)').exec(location.search) || [, null])[1]
+			);
+}
+
 /**
  * takes an absolute timestamp and return a string with a human-friendly relative date
  * @param int a Unix timestamp
@@ -782,15 +931,13 @@ function relative_modified_date(timestamp) {
 	var diffdays = Math.round(diffhours/24);
 	var diffmonths = Math.round(diffdays/31);
 	if(timediff < 60) { return t('core','seconds ago'); }
-	else if(timediff < 120) { return t('core','1 minute ago'); }
-	else if(timediff < 3600) { return t('core','{minutes} minutes ago',{minutes: diffminutes}); }
-	else if(timediff < 7200) { return t('core','1 hour ago'); }
-	else if(timediff < 86400) { return t('core','{hours} hours ago',{hours: diffhours}); }
+	else if(timediff < 3600) { return n('core','%n minute ago', '%n minutes ago', diffminutes); }
+	else if(timediff < 86400) { return n('core', '%n hour ago', '%n hours ago', diffhours); }
 	else if(timediff < 86400) { return t('core','today'); }
 	else if(timediff < 172800) { return t('core','yesterday'); }
-	else if(timediff < 2678400) { return t('core','{days} days ago',{days: diffdays}); }
+	else if(timediff < 2678400) { return n('core', '%n day ago', '%n days ago', diffdays); }
 	else if(timediff < 5184000) { return t('core','last month'); }
-	else if(timediff < 31556926) { return t('core','{months} months ago',{months: diffmonths}); }
+	else if(timediff < 31556926) { return n('core', '%n month ago', '%n months ago', diffmonths); }
 	//else if(timediff < 31556926) { return t('core','months ago'); }
 	else if(timediff < 63113852) { return t('core','last year'); }
 	else { return t('core','years ago'); }
@@ -804,7 +951,7 @@ OC.get=function(name) {
 	var namespaces = name.split(".");
 	var tail = namespaces.pop();
 	var context=window;
-	
+
 	for(var i = 0; i < namespaces.length; i++) {
 		context = context[namespaces[i]];
 		if(!context){
@@ -823,7 +970,7 @@ OC.set=function(name, value) {
 	var namespaces = name.split(".");
 	var tail = namespaces.pop();
 	var context=window;
-	
+
 	for(var i = 0; i < namespaces.length; i++) {
 		if(!context[namespaces[i]]){
 			context[namespaces[i]]={};
@@ -839,7 +986,7 @@ OC.set=function(name, value) {
  * @param {type} start
  * @param {type} end
  */
-$.fn.selectRange = function(start, end) {
+jQuery.fn.selectRange = function(start, end) {
 	return this.each(function() {
 		if (this.setSelectionRange) {
 			this.focus();
@@ -855,14 +1002,11 @@ $.fn.selectRange = function(start, end) {
 };
 
 /**
- * Calls the server periodically every 15 mins to ensure that session doesnt
- * time out
+ * check if an element exists.
+ * allows you to write if ($('#myid').exists()) to increase readability
+ * @link http://stackoverflow.com/questions/31044/is-there-an-exists-function-for-jquery
  */
-function sessionHeartBeat(){
-	OC.Router.registerLoadedCallback(function(){
-		var url = OC.Router.generate('heartbeat');
-		setInterval(function(){
-			$.post(url);
-		}, 900000);
-	});
-}
+jQuery.fn.exists = function(){
+	return this.length > 0;
+};
+
