@@ -25,7 +25,7 @@
 			'labels':[],
 			'oncheck':false,
 			'onuncheck':false,
-			'minWidth': 'default;',
+			'minWidth': 'default;'
 		};
 		$(this).attr('data-msid', multiSelectId);
 		$.extend(settings,options);
@@ -156,8 +156,6 @@
 					var newOuterWidth=Math.max((button.outerWidth()-2),settings.minOuterWidth)+'px';
 					var newWidth=Math.max(button.width(),settings.minWidth);
 					var pos=button.position();
-					button.css('height',button.height());
-					button.css('white-space','nowrap');
 					button.css('width',oldWidth);
 					button.animate({'width':newWidth},undefined,undefined,function(){
 						button.css('width','');
@@ -176,10 +174,10 @@
 			});
 			button.parent().data('preventHide',false);
 			if(settings.createText){
-				var li=$('<li class="creator">+ <em>'+settings.createText+'<em></li>');
+				var li=$('<li class="creator">+ '+settings.createText+'</li>');
 				li.click(function(event){
 					li.empty();
-					var input=$('<input class="new">');
+					var input=$('<input type="text" class="new">');
 					li.append(input);
 					input.focus();
 					input.css('width',button.innerWidth());
@@ -200,7 +198,7 @@
 								return false;
 							}
 							var li=$(this).parent();
-							var val = $(this).val()
+							var val = $(this).val();
 							var select=button.parent().next();
 							if(typeof settings.createCallback === 'function') {
 								var response = settings.createCallback(select, val);
@@ -316,4 +314,4 @@
 		
 		return span;
 	};
-})( jQuery );
\ No newline at end of file
+})( jQuery );
